"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const type_analyzer_1 = require("@previewjs/type-analyzer");
const vfs_1 = require("@previewjs/vfs");
const path_1 = __importDefault(require("path"));
const vitest_1 = require("vitest");
const _1 = require(".");
const extract_component_1 = require("./extract-component");
const ROOT_DIR = path_1.default.join(__dirname, "virtual");
const MAIN_FILE = path_1.default.join(ROOT_DIR, "App.tsx");
const STORIES_FILE = path_1.default.join(ROOT_DIR, "App.stories.tsx");
(0, vitest_1.describe)("extractReactComponents", () => {
    let memoryReader;
    let typeAnalyzer;
    (0, vitest_1.beforeEach)(async () => {
        memoryReader = (0, vfs_1.createMemoryReader)();
        memoryReader.updateFile(MAIN_FILE, "export default () => <div>Hello, World!</div>;");
        const frameworkPlugin = await _1.reactFrameworkPlugin.create();
        typeAnalyzer = (0, type_analyzer_1.createTypeAnalyzer)({
            rootDirPath: ROOT_DIR,
            reader: (0, vfs_1.createStackedReader)([
                memoryReader,
                (0, vfs_1.createFileSystemReader)({
                    watch: false,
                }), // required for TypeScript libs, e.g. Promise
            ]),
            tsCompilerOptions: frameworkPlugin.tsCompilerOptions,
        });
    });
    (0, vitest_1.afterEach)(() => {
        typeAnalyzer.dispose();
    });
    (0, vitest_1.it)("detects expected components", async () => {
        memoryReader.updateFile(MAIN_FILE, `
import React, { Component, PureComponent } from "react";

function wrapComponent<P>(c: React.ComponentType<P>): React.ComponentType<P> {
  return c;
}

function DeclaredFunction() {
  return <div>Hello, World!</div>;
}

const ConstantFunction = () => <div>Hello, World!</div>;

// Note: this isn't detected as of October 2021.
const HocComponent = wrapComponent(() => <div>Hello, World!</div>);

class ClassComponent1 extends React.Component {}

class ClassComponent2 extends Component {}

const ForwardRef = React.forwardRef((props, ref) => <ConstantFunction {...props} />);

const NextComponent: NextPage = () => {
  return <div>Hello, World!</div>;
};

export class Pure extends PureComponent<{foo: string}> {}

export class BaseClass<T> {}

export class NotComponentClass extends BaseClass<SomeProps> {}

export const NotObjectProps = (props: () => void) => {
  return <div>Hello, World!</div>;
};

export const MissingType = (props) => {
  return <div>Hello, World!</div>;
};

export const NotComponent = () => {
  // This isn't a component.
};

export const NotComponentEither = () => {
  return "Hello";
};

export const AlsoNotAStory = {
  args: {}
};
`);
        (0, vitest_1.expect)(extract(MAIN_FILE)).toMatchObject([
            {
                name: "DeclaredFunction",
                info: {
                    kind: "component",
                    exported: false,
                },
            },
            {
                name: "ConstantFunction",
                info: {
                    kind: "component",
                    exported: false,
                },
            },
            // Note: this isn't detected as of October 2021.
            // {
            //   name: "HocComponent",
            //   exported: false,
            //   offsets: expect.anything(),
            // },
            {
                name: "ClassComponent1",
                info: {
                    kind: "component",
                    exported: false,
                },
            },
            {
                name: "ClassComponent2",
                info: {
                    kind: "component",
                    exported: false,
                },
            },
            {
                name: "ForwardRef",
                info: {
                    kind: "component",
                    exported: false,
                },
            },
            {
                name: "NextComponent",
                info: {
                    kind: "component",
                    exported: false,
                },
            },
            {
                name: "Pure",
                info: {
                    kind: "component",
                    exported: true,
                },
            },
            {
                name: "NotObjectProps",
                info: {
                    kind: "component",
                    exported: true,
                },
            },
            {
                name: "MissingType",
                info: {
                    kind: "component",
                    exported: true,
                },
            },
        ]);
    });
    (0, vitest_1.it)("detects components without any React import", async () => {
        memoryReader.updateFile(MAIN_FILE, `
function DeclaredFunction() {
  return <div>Hello, World!</div>;
}

const ConstantFunction = () => <div>Hello, World!</div>;
`);
        (0, vitest_1.expect)(extract(MAIN_FILE)).toMatchObject([
            {
                name: "DeclaredFunction",
                info: {
                    kind: "component",
                    exported: false,
                },
            },
            {
                name: "ConstantFunction",
                info: {
                    kind: "component",
                    exported: false,
                },
            },
        ]);
    });
    (0, vitest_1.it)("ignores default export of identifier", async () => {
        memoryReader.updateFile(MAIN_FILE, `
const A = () => {
  return <div>Hello, World!</div>;
};

export default A;
`);
        (0, vitest_1.expect)(extract(MAIN_FILE)).toMatchObject([
            {
                name: "A",
                info: {
                    kind: "component",
                    exported: true,
                },
            },
        ]);
    });
    (0, vitest_1.it)("detects default export component (arrow function)", async () => {
        memoryReader.updateFile(MAIN_FILE, `
export default () => {
  return <div>Hello, World!</div>;
}
`);
        (0, vitest_1.expect)(extract(MAIN_FILE)).toMatchObject([
            {
                name: "default",
                info: {
                    kind: "component",
                    exported: true,
                },
            },
        ]);
    });
    (0, vitest_1.it)("detects default export component (named function)", async () => {
        memoryReader.updateFile(MAIN_FILE, `
export default function test(){
  return <div>Hello, World!</div>;
}
`);
        (0, vitest_1.expect)(extract(MAIN_FILE)).toMatchObject([
            {
                name: "test",
                info: {
                    kind: "component",
                    exported: true,
                },
            },
        ]);
    });
    (0, vitest_1.it)("detects default export component (anonymous function)", async () => {
        memoryReader.updateFile(MAIN_FILE, `
export default function(){
  return <div>Hello, World!</div>;
}
`);
        (0, vitest_1.expect)(extract(MAIN_FILE)).toMatchObject([
            {
                name: "default",
                info: {
                    kind: "component",
                    exported: true,
                },
            },
        ]);
    });
    (0, vitest_1.it)("does not detect default export non-component", async () => {
        memoryReader.updateFile(MAIN_FILE, `
export default () => {
  return "foo";
}
`);
        (0, vitest_1.expect)(extract(MAIN_FILE)).toMatchObject([]);
    });
    (0, vitest_1.it)("detects CSF2 stories", async () => {
        memoryReader.updateFile(STORIES_FILE, `
import Button from "./App";

export default {
  component: Button
}

const Template = (args) => <Button {...args} />;

export const Primary = Template.bind({});
Primary.args = {
   primary: true,
   label: 'Button',
};
`);
        (0, vitest_1.expect)(extract(STORIES_FILE)).toMatchObject([
            {
                name: "Template",
                info: {
                    kind: "component",
                    exported: false,
                },
            },
            {
                name: "Primary",
                info: {
                    kind: "story",
                    associatedComponent: {
                        absoluteFilePath: MAIN_FILE,
                        name: "default",
                    },
                },
            },
        ]);
    });
    (0, vitest_1.it)("detects CSF3 stories", async () => {
        memoryReader.updateFile(STORIES_FILE, `
import Button from "./App";

export default {
  component: Button
}

export const Example = {
  args: {
    label: "Hello, World!"
  }
}

export const NoArgs = {}

export function NotStory() {}
`);
        (0, vitest_1.expect)(extract(STORIES_FILE)).toMatchObject([
            {
                name: "Example",
                info: {
                    kind: "story",
                    associatedComponent: {
                        absoluteFilePath: MAIN_FILE,
                        name: "default",
                    },
                },
            },
            {
                name: "NoArgs",
                info: {
                    kind: "story",
                    associatedComponent: {
                        absoluteFilePath: MAIN_FILE,
                        name: "default",
                    },
                },
            },
        ]);
    });
    function extract(absoluteFilePath) {
        return (0, extract_component_1.extractReactComponents)(typeAnalyzer.analyze([absoluteFilePath]), absoluteFilePath);
    }
});
